=== modified file 'NEWS'
--- NEWS	2007-09-12 21:27:42 +0000
+++ NEWS	2007-09-13 23:16:40 +0000
@@ -17,6 +17,16 @@
 
   BUG FIXES:
 
+   * The basename, not the full path, is now used when checking whether
+     the profiling dump file begins with ``callgrind.out`` or not. This
+     fixes a bug reported by Aaron Bentley on IRC. (Ian Clatworthy)
+
+   * Commands reporting exceptions can now be profiled and still have their
+     data correctly dumped to a file. For example, a ``bzr commit`` with
+     no changes still reports the operation as pointless but doing so no
+     longer throws away the profiling data if this command is run with
+     ``--lsprof-file callgrind.out.ci`` say. (Ian Clatworthy)
+
   API BREAKS:
 
    * The ``VersionedFile`` interface now allows content checks to be bypassed

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2007-09-05 08:18:57 +0000
+++ bzrlib/commands.py	2007-09-10 02:40:07 +0000
@@ -796,6 +796,8 @@
 
 
 def run_bzr_catch_errors(argv):
+    # Note: The except clause logic below should be kept in sync with the
+    # profile() routine in lsprof.py.
     try:
         return run_bzr(argv)
     except (KeyboardInterrupt, Exception), e:

=== modified file 'bzrlib/lsprof.py'
--- bzrlib/lsprof.py	2007-07-30 01:55:44 +0000
+++ bzrlib/lsprof.py	2007-09-10 08:19:29 +0000
@@ -10,6 +10,10 @@
 import threading
 from _lsprof import Profiler, profiler_entry
 
+
+import bzrlib.osutils
+
+
 __all__ = ['profile', 'Stats']
 
 _g_threadmap = {}
@@ -31,8 +35,17 @@
     p = Profiler()
     p.enable(subcalls=True)
     threading.setprofile(_thread_profile)
+    # Note: The except clause is needed below so that profiling data still
+    # gets dumped even when exceptions are encountered. The except clause code
+    # is taken straight from run_bzr_catch_errrors() in commands.py and ought
+    # to be kept in sync with it.
     try:
-        ret = f(*args, **kwds)
+        try:
+            ret = f(*args, **kwds)
+        except (KeyboardInterrupt, Exception), e:
+            import bzrlib.trace
+            bzrlib.trace.report_exception(sys.exc_info(), sys.stderr)
+            ret = 3
     finally:
         p.disable()
         for pp in _g_threadmap.values():
@@ -118,10 +131,11 @@
             otherwise the format is given by the filename extension.
         """
         if format is None:
-            if filename.startswith('callgrind.out'):
+            basename = bzrlib.osutils.basename(filename)
+            if basename.startswith('callgrind.out'):
                 format = "callgrind"
             else:
-                ext = os.path.splitext(filename)[1]
+                ext = bzrlib.osutils.splitext(filename)[1]
                 if len(ext) > 1:
                     format = ext[1:]
         outfile = open(filename, 'wb')

=== modified file 'bzrlib/tests/test_lsprof.py'
--- bzrlib/tests/test_lsprof.py	2007-06-18 13:55:14 +0000
+++ bzrlib/tests/test_lsprof.py	2007-09-10 06:27:40 +0000
@@ -64,7 +64,7 @@
 
     def _tempfile(self, ext):
         dir = self.test_dir
-        return os.path.join(dir, "tmp_profile_data." + ext)
+        return bzrlib.osutils.pathjoin(dir, "tmp_profile_data." + ext)
 
     def test_stats_save_to_txt(self):
         f = self._tempfile("txt")
@@ -77,6 +77,10 @@
         self.stats.save(f)
         lines = open(f).readlines()
         self.assertEqual(lines[0], "events: Ticks\n")
+        f = bzrlib.osutils.pathjoin(self.test_dir, "callgrind.out.foo")
+        self.stats.save(f)
+        lines = open(f).readlines()
+        self.assertEqual(lines[0], "events: Ticks\n")
         # Test explicit format nommination
         f2 = self._tempfile("txt")
         self.stats.save(f2, format="callgrind")

